__version__ = '0.1.13'
__license__ = 'GNU Affero General Public License v3.0'
__description__ = 'Cadasta Worker Toolbox'
__author__ = 'Anthony Lukach'
__email__ = 'alukach@cadasta.org'
__url__ = 'https://github.com/Cadasta/cadasta-workertoolbox'


DEFAULT_QUEUES = (
    'msg',
    'export',
)
